/**
 * apeman app to send html response.
 * @function apemanAppHtml
 * @param {object} [options] - Optional settings.
 * @param {string} [options.contentType='application/html; charset=UTF-8;'] - Content type for header
 * @param {string} [options.name='html'] - Property name of response to set the function.
 * @returns {function} - Defined app function.
 */

'use strict'

const argx = require('argx')
const objnest = require('objnest')

/** @lends create */
function create (options) {
  let args = argx(arguments)
  options = objnest.expand(args.pop('object') || {})

  let contentType = options.contentType || 'text/html; charset=UTF-8;'
  let name = options.name || 'html'

  function html (data) {
    if (typeof data === 'object') {
      try {
        data = JSON.stringify(data)
      } catch (e) {
        // Do nothing
      }
    }
    let res = this
    let body = String(data).trim()
    let hasDocType = /<!DOCTYPE\shtml>/.test(body)
    if (!hasDocType) {
      body = '<!DOCTYPE html>' + body
    }
    res.setHeader('Content-Length', Buffer.byteLength(body))
    res.setHeader('Content-Type', contentType)
    res.end(body)
  }

  /**
   * Defined app.
   * @function app
   * @param {object} req - Request object.
   * @param {object} res - Response object.
   * @param {function} next - Pass to next handler.
   */
  function app (req, res, next) {
    res[ name ] = res[ name ] || html
    next()
  }

  Object.assign(app, {
    // Description of this app.
    $desc: 'Define html response function.'
  })
  return app
}

module.exports = create
